<?php
/***************************************************************
 *  Copyright notice
 *
 *  (c) 2004 Robert Lemke (robert@typo3.org)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
* 
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
* 
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/

require_once (TX_PHPUNITLIB_EXTPATH.'Framework/TestListener.php');

class tx_phpunit_testlistener implements PHPUnit_Framework_TestListener {

	protected	$resultArr = array();
	public		$totalNumberOfTestCases = 0;				// Set from outside
	protected	$currentTestNumber = 0;						// For counting the tests		
	
	/**
	 *	An error occurred.
	 *
	 *	@param  PHPUnit_Framework_Test $test
	 *	@param  Exception               $e
	 *	@access public
	 */
    public function addError(PHPUnit_Framework_Test $test, Exception $e, $time) {
    	$testCaseTraceArr = $this->getFirstNonPHPUnitTrace ($e->getTrace());
		$fileName = str_replace (PATH_site, '', $testCaseTraceArr['file']);		
    	echo '
			<script>document.getElementById("progress-bar").style.backgroundColor = "#AA0000";</script>	
			<br />
			<span style="color:red;">!</span> <strong>Error</strong> in test case <em>'.$test->getName().'</em> in file <em>'.$fileName.'</em> line <em>'.$testCaseTraceArr['line'].'</em>:<br />
			'.$e->getMessage().'<br />
		';
    	flush();
    }

    /**
     * A failure occurred.
     *
     * @param  PHPUnit_Framework_Test                 $test
     * @param  PHPUnit_Framework_AssertionFailedError $e
     * @access public
     */
    public function addFailure(PHPUnit_Framework_Test $test, PHPUnit_Framework_AssertionFailedError $e, $time) {
    	$testCaseTraceArr = $this->getFirstNonPHPUnitTrace ($e->getTrace());
		$fileName = str_replace (PATH_site, '', $testCaseTraceArr['file']);		
    	echo '
			<script>document.getElementById("progress-bar").style.backgroundColor = "#AA0000";</script>	
			<br />
			<span style="color:red;">!</span> <strong>Failure</strong> in test case <em>'.$test->getName().'</em> in file <em>'.$fileName.'</em> line <em>'.$testCaseTraceArr['line'].'</em>:<br />
			'.$e->getMessage().'<br />
		';
		flush();
    }

    /**
    * Incomplete test.
    *
    * @param  PHPUnit_Framework_Test $test
    * @param  Exception               $e
    * @access public
    */
    public function addIncompleteTest(PHPUnit_Framework_Test $test, Exception $e, $time) {
    	echo '
			<br />
			<span style="color:grey;">!</span> <strong>Incomplete test</strong> <em>'.$test->getName().'</em> in file <em>'.$e->getFile().'</em> line <em>'.$e->getLine().'</em>:<br />
			'.$e->getMessage().'<br />
		';
		flush();
    }

    
    /**
     * Skipped test.
     *
     * @param  PHPUnit_Framework_Test $test
     * @param  Exception              $e
     * @param  float                  $time
     * @access public
     * @since  Method available since Release 3.0.0
     */
    public function addSkippedTest(PHPUnit_Framework_Test $test, Exception $e, $time) {
    	// TODO: Implement addSkippedTest
    }
    
    /**
    * A testsuite started.
    *
    * @param  PHPUnit_Framework_TestSuite $suite
    * @access public
    */
    public function startTestSuite(PHPUnit_Framework_TestSuite $suite) {
		echo ('<strong>'.$suite->getName().'</strong>:<br/>');
    }

    /**
    * A testsuite ended.
    *
    * @param  PHPUnit_Framework_TestSuite $suite
    * @access public
    */
    public function endTestSuite(PHPUnit_Framework_TestSuite $suite) {
    }

    /**
    * A test started.
    *
    * @param  PHPUnit_Framework_Test $test
    * @access public
    */
    public function startTest(PHPUnit_Framework_Test $test) {
    }

    /**
     * A test ended.
     *
     * @param  PHPUnit_Framework_Test $test
     * @access public
     */
    public function endTest(PHPUnit_Framework_Test $test, $time) {
    	$this->currentTestNumber ++;
    	$percentDone = intval (($this->currentTestNumber / $this->totalNumberOfTestCases) * 100);
    	
    	echo '<script>document.getElementById("progress-bar").style.width = "'.$percentDone.'%";</script>';
    	echo '<script>document.getElementById("transparent-bar").style.width = "'.(100-$percentDone).'%";</script>';    	
  
  		echo '-> '.$test->getName().'<br />';  	
     	flush();
    }

    /**
     * Returns the first trace information which is not caused by the PHPUnit file
     * "Framework/Assert.php".
     *
     * @param 	array	$traceArr: The trace array
     * @return	array	Trace information
     * @access 	protected
     */
	protected function getFirstNonPHPUnitTrace ($traceArr) {
		$testCaseTraceArr = array();
		foreach ($traceArr as $singleTraceArr) {
			if (!stristr ($singleTraceArr['file'], 'Framework/Assert.php')) {
				$testCaseTraceArr = $singleTraceArr;
				break;	
			}
		}
		return $testCaseTraceArr;
	}
}


?>