<?php
/***************************************************************
*  Copyright notice
*
*  (c) 2004-2007 Kasper Ligaard (ligaard@daimi.au.dk)
*  All rights reserved
*
*  This script is part of the TYPO3 project. The TYPO3 project is
*  free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  The GNU General Public License can be found at
*  http://www.gnu.org/copyleft/gpl.html.
*
*  This script is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  This copyright notice MUST APPEAR in all copies of the script!
***************************************************************/
/**
 * Module 'PHPUnit' for the 'phpunit' extension.
 *
 * @author	Kasper Ligaard <ligaard@daimi.au.dk>
 */

/**
 * [CLASS/FUNCTION INDEX of SCRIPT]
 *
 *
 *
 *   85: class tx_phpunitt3_module1 extends t3lib_SCbase
 *   93:     public function menuConfig()
 *  112:     public function main()
 *
 *              SECTION: Screen render functions
 *  179:     protected function runTests_render()
 *  200:     protected function runTests_renderIntro()
 *  223:     protected function runTests_renderIntro_renderExtensionSelector($extensionsWithTestSuites)
 *  258:     protected function runTests_renderIntro_renderTestSelector($extensionsWithTestSuites, $extensionKey)
 *  303:     protected function runTests_renderRunningTest()
 *  387:     protected function runTests_renderInfoAndProgressbar()
 *  403:     protected function about_render()
 *
 *              SECTION: Helper functions
 *  437:     protected function openNewWindowLink()
 *  456:     protected function getExtensionsWithTestSuites()
 *  466:     protected function traversePathForTestCases($path)
 *  502:     protected function simulateFrontendEnviroment()
 *
 * TOTAL FUNCTIONS: 13
 * (This index is automatically created/updated by the extension "extdeveval")
 *
 */

unset($MCONF);
require ('conf.php');
require ($BACK_PATH.'init.php');
require ($BACK_PATH.'template.php');

$LANG->includeLLFile('EXT:phpunit/mod1/locallang.php');

require_once (PATH_t3lib.'class.t3lib_scbase.php');
require_once (t3lib_extMgm::extPath('phpunit').'phpunit-3.0.5/Framework/TestSuite.php');

require_once (t3lib_extMgm::extPath('phpunit').'class.tx_phpunit_testlistener.php');
require_once (t3lib_extMgm::extPath('phpunit').'class.tx_phpunit_testcase.php');

define('PATH_tslib', t3lib_extMgm::extPath('cms').'tslib/');

/**
 * Module 'PHPUnit' for the 'phpunit' extension.
 *
 * @author	Kasper Ligaard <ligaard@daimi.au.dk>
 * @package TYPO3
 * @subpackage tx_phpunit
 */
class tx_phpunit_module1 extends t3lib_SCbase {

	/**
	 * Create configuration for the function selector box
	 *
	 * @return	void
	 * @access	public
	 */
	public function menuConfig()	{
		global $LANG;
		$this->MOD_MENU = Array (
			'function' => Array (
				'runtests' => $LANG->getLL('function_runtests'),
				'about' => $LANG->getLL('function_about'),
			),
			'extSel' => '',
		);
		parent::menuConfig();
	}

	/**
	 * Main function of the module. All content is echoed directly instead of collecting it and
	 * doing the output later.
	 *
	 * @return	void
	 * @access	public
	 */
	public function main()	{
		global $BE_USER,$LANG,$BACK_PATH,$TCA_DESCR,$TCA,$CLIENT,$TYPO3_CONF_VARS;

		if ($BE_USER->user['admin']) {

				// Draw the header.
			$this->doc = t3lib_div::makeInstance('bigDoc');
			$this->doc->backPath = $BACK_PATH;

				// JavaScript
			$this->doc->JScode = $this->doc->wrapScriptTags('
					script_ended = 0;
					function jumpToUrl(URL)	{	//
						document.location = URL;
					}
			');

			echo $this->doc->startPage($LANG->getLL('title'));
			echo $this->doc->header($LANG->getLL('title'));
			echo $this->doc->spacer(5);
			echo $this->doc->section('',$this->doc->funcMenu('', t3lib_BEfunc::getFuncMenu($this->id,'SET[function]',$this->MOD_SETTINGS['function'],$this->MOD_MENU['function']).'&nbsp;&nbsp;&nbsp;'.$this->openNewWindowLink()));
			echo $this->doc->divider(5);

				// Render content:
			switch ($this->MOD_SETTINGS['function']) {
				case 'runtests' :
					$this->runTests_render();
					break;
				case 'about' :
					$this->about_render();
					break;
			}

				// ShortCut
			echo $this->doc->spacer(20).$this->doc->section('',$this->doc->makeShortcutIcon('id',implode(',',array_keys($this->MOD_MENU)),$this->MCONF['name']));
			echo $this->doc->spacer(10);

		} else {

			$this->doc = t3lib_div::makeInstance('mediumDoc');
			$this->doc->backPath = $BACK_PATH;

			echo $this->doc->startPage($LANG->getLL('title'));
			echo $this->doc->header($LANG->getLL('title'));
			echo $this->doc->spacer(5);
			echo $this->doc->spacer(10);
			echo  'You need administrator rights in order to use PHPUnit Backend Module.';
		}
		echo $this->doc->endPage();
	}





	/*********************************************************
	 *
	 * 	Screen render functions
	 *
	 *********************************************************/

	/**
	 * Renders the screens for function "Run tests"
	 *
	 * @return	void
	 * @access	protected
	 */
	protected function runTests_render() {

		$command = $this->MOD_SETTINGS['extSel'] ? t3lib_div::_GP('command') : '';

		switch ($command) {
			case 'runalltests':
			case 'runsingletest':
				$this->runTests_renderIntro();
				$this->runTests_renderRunningTest();
				break;
			default:
				$this->runTests_renderIntro();
			}
	}

	/**
	 * Renders the intro screen for the function "Run tests"
	 *
	 * @return	void
	 * @access	protected
	 */
	protected function runTests_renderIntro() {
		$output = '';

		$extensionsWithTestSuites = $this->getExtensionsWithTestSuites();
		if (is_array($extensionsWithTestSuites))	{
			ksort($extensionsWithTestSuites);

			$output = $this->runTests_renderIntro_renderExtensionSelector($extensionsWithTestSuites);
			if ($this->MOD_SETTINGS['extSel'] && $this->MOD_SETTINGS['extSel']!='uuall') {
				$output .= $this->runTests_renderIntro_renderTestSelector($extensionsWithTestSuites, $this->MOD_SETTINGS['extSel']);
			}
		} else $output = 'Could not find any extension with unit tests.';

		echo $output;
	}

	/**
	 * Renders the the extension selectorbox
	 *
	 * @param	array		$extensionsWithTestSuites: Array of extension keys for which test suites exist
	 * @return	string		HTML code for the selectorbox and a surrounding form
	 * @access	protected
	 */
	protected function runTests_renderIntro_renderExtensionSelector($extensionsWithTestSuites) {

		$extensionsOptionsArr =array();
		$extensionsOptionsArr[] = '<option value="">[ Select Extension ]</option>';

		$selected = strcmp('uuall',$this->MOD_SETTINGS['extSel']) ? '' : ' selected="selected"';
		$extensionsOptionsArr[] ='<option value="uuall"'.$selected.'>All Extensions</option>';

		$extensionsOptionsArr[] ='<option value="">--------------------</option>';

		foreach($extensionsWithTestSuites as $dirName => $dummy)		{
			$selected = strcmp($dirName,$this->MOD_SETTINGS['extSel']) ? '' : ' selected="selected"';
			$extensionsOptionsArr[]='<option value="'.htmlspecialchars($dirName).'"'.$selected.'>'.htmlspecialchars($dirName).'</option>';
		}

		$output = '
			<form action="index.php" method="POST">
				<select name="SET[extSel]" onchange="jumpToUrl(\'?SET[extSel]=\'+this.options[this.selectedIndex].value,this);">'.implode('',$extensionsOptionsArr).'</select>
				<input type="submit" value="Run all tests" />
				<input name="command" type="hidden" value="runalltests" />
			</form>
			<br />
		';

		return $output;
	}

	/**
	 * Renders a selector box for running single tests for the given extension
	 *
	 * @param	array		$extensionsWithTestSuites: Array of extension keys for which test suites exist
	 * @param	string		$extensionKey: Extension key of the extensino to run single test for
	 * @return	string		HTML code with the selectorbox and a surrounding form
	 * @access	protected
	 */
	protected function runTests_renderIntro_renderTestSelector($extensionsWithTestSuites, $extensionKey) {

		$testSuite = new PHPUnit_Framework_TestSuite('tx_phpunit_basetestsuite');

			// load the files containing test cases from extensions:
		$infoArr = $extensionsWithTestSuites[$extensionKey];
		foreach ($infoArr['files'] as $fileName) {
			require_once ($infoArr['path'].$fileName);
		}

			// Add all classes to the test suite which end with "testcase"
		foreach (get_declared_classes() as $class) {
			if (substr ($class, -8, 8) == 'testcase' && $class != 'tx_phpunit_testcase') {
				$testSuite->addTestSuite ($class);
			}
		}

		$testsOptionsArr =array();
		$testsOptionsArr[] = '<option value="">[ Select test ]</option>';

		foreach ($testSuite->tests() as $testCases) {
			foreach ($testCases->tests() as $test) {
				$selected = $test->getName() == t3lib_div::GPvar('testname') ? ' selected="selected"' : '';
				$testsOptionsArr[] = '<option value="'.$test->getName().'"'.$selected.'>'.htmlspecialchars($test->getName()).'</option>';
			}
		}

		$output = '
			<form action="index.php" method="post">
				<select name="testname">'.implode('',$testsOptionsArr).'</select>
				<input type="submit" value="Run single test" />
				<input type="hidden" name="command" value="runsingletest" />
			</form>
		';

		return $output;
	}

	/**
	 * Renders the screen for the function "Run tests" which shows and
	 * runs the actual unit tests
	 *
	 * @return	void
	 * @access	protected
	 */
	protected function runTests_renderRunningTest() {

		$this->simulateFrontendEnviroment();

		$extensionsWithTestSuites = $this->getExtensionsWithTestSuites();
		$testSuite = new PHPUnit_Framework_TestSuite('tx_phpunit_basetestsuite');

		if($this->MOD_SETTINGS['extSel']=='uuall') {
			echo '<h3>Testing all Extensions</h3>';
		} else {
			echo '<h3>Testing Extension '.htmlspecialchars($this->MOD_SETTINGS['extSel']).'</h3>';
			$extInfo = $extensionsWithTestSuites[$this->MOD_SETTINGS['extSel']];
			$extensionsWithTestSuites = array();
			$extensionsWithTestSuites[$this->MOD_SETTINGS['extSel']] = $extInfo;
		}

		$this->runTests_renderInfoAndProgressbar();

			// load the files containing test cases from extensions:
		foreach ($extensionsWithTestSuites as $extensionKey => $infoArr) {
			foreach ($infoArr['files'] as $fileName) {
				require_once ($infoArr['path'].$fileName);
			}
		}

			// Add all classes to the test suite which end with "testcase"
		foreach (get_declared_classes() as $class) {
			if (substr ($class, -8, 8) == 'testcase' && $class != 'tx_phpunit_testcase') {
				$testSuite->addTestSuite ($class);
			}
		}

			// Create a listener and run the tests:
		$testListener = new tx_phpunit_testlistener;

		$testResult = new PHPUnit_Framework_TestResult;
		$testResult->addListener ($testListener);

		if (t3lib_div::GPvar('testname')) {
			$testListener->totalNumberOfTestCases = 1;
			foreach ($testSuite->tests() as $testCases) {
				foreach ($testCases->tests() as $test) {
					if ($test->getName() == t3lib_div::GPvar('testname')) {
						$test->run ($testResult);
					}
				}
			}
		} else {
			$testListener->totalNumberOfTestCases = $testSuite->testCount();
			$testSuite->run ($testResult);
		}

			// Display test statistics:
		if ($testResult->wasSuccessful()) {
	    	echo '
				<script>document.getElementById("progress-bar").style.backgroundColor = "green";</script>
				<br /><span style="color:green;"><strong>SUCCESS!</strong></span><br />
				'.$testResult->count().' tests, '.$testResult->failureCount().' failures, '.$testResult->errorCount().' errors.<br />
			';
		} else {
	    	echo '
				<script>document.getElementById("progress-bar").style.backgroundColor = "red";</script>
				<br /><span style="color:red;"><strong>FAILURES!</strong></span><br />
				'.$testResult->count().' tests, '.$testResult->failureCount().' failures, '.$testResult->errorCount().' errors.<br />
			';
		}

		echo '
			<form action="index.php" method="POST" >
				<input type="submit" value="Run again" tabindex="100" />
				<input name="command" type="hidden" value="'.t3lib_div::_GP('command').'" />
				<input name="testname" type="hidden" value="'.t3lib_div::_GP('testname').'" />
			</form>
		';
	}

	/**
	 * Renders DIVs which contain information and a progressbar to visualize
	 * the running tests. The actual information will be written via JS during
	 * the test runs.
	 *
	 * @return	void
	 * @access	protected
	 */
	protected function runTests_renderInfoAndProgressbar() {
		echo '
			<div style="width:100%; height:20px; border: 1px solid black;">
				<div id="progress-bar" style="float: left; width: 40%; height: 20px; background-color:grey;">&nbsp;</div>
				<div id="transparent-bar" style="float: left; width: 60%; height: 20px; background-color:'.$parentObj->doc->bgColor2.';">&nbsp;</div>
			</div>
			<br />
		';
	}

	/**
	 * Renders the "About" screen
	 *
	 * @return	void
	 * @access	protected
	 */
	protected function about_render() {
		echo '
			<h3>About PHPUnit Backend Module</h3>
			<div style="height: 90px;">
				<img src="phpunit.gif" width="94" height="80" alt="PHPUnit" title="PHPUnit" style="float:right; margin-left:10px;" />
				PHPUnit BE is a unit testing framework based on <a href="http://www.phpunit.de/" target="_new">PHPUnit</a> by Sebastian Bergmann. It offers smooth integration
				of the PHPUnit framework into TYPO3 and offers an API and many functions which make unit testing of TYPO3 extensions easy and comfortable.<br />
				<br />
			</div>
			<h3>Licence and copyright</h3>
			PHPUnit is released under the terms of the PHP License as free software.<br />
			PHPUnit Copyright &copy; 2001 - 2006 Sebastian Bergmann<br />
			<br />
			PHPUnit BE is released under the GPL Licence and is part of the TYPO3 Framework.<br />
			PHPUnit BE Copyright &copy; 2005-2006 <a href="mailto:robert@typo3.org">Robert Lemke</a><br />
		';
	}





	/*********************************************************
	 *
	 * 	Helper functions
	 *
	 *********************************************************/

	/**
	 * Renders a link which opens the current screen in a new window
	 *
	 * @return	string
	 * @access	protected
	 */
	protected function openNewWindowLink()	{
		global $LANG, $BACK_PATH;

		$url = t3lib_div::getIndpEnv('TYPO3_REQUEST_SCRIPT');
		$onClick = "phpunitbeWin=window.open('".$url."','phpunitbe','width=790,status=0,menubar=1,resizable=1,location=0,scrollbars=1,toolbar=0');phpunitbeWin.focus();return false;";
		$content = '
			<a id="opennewwindow" href="#" onclick="'.htmlspecialchars($onClick).'">
				<img'.t3lib_iconWorks::skinImg ($BACK_PATH,'gfx/open_in_new_window.gif','width="19" height="14"').' title="'.$LANG->sL('LLL:EXT:lang/locallang_core.php:labels.openInNewWindow',1).'" class="absmiddle" '.$addAttrib.' alt="" />
			</a>
			<script language="JavaScript"> if(window.name=="phpunitbe") { document.getElementById("opennewwindow").style.display = "none"; } </script>
		';
		return $content;
	}

	/**
	 * Scans all available extensions for test suites and returns the path / file names in an array
	 *
	 * @return	array		Array of testcase files
	 */
	protected function getExtensionsWithTestSuites() {
		return array_merge ($this->traversePathForTestCases (PATH_site.'typo3conf/ext/'), $this->traversePathForTestCases (PATH_site.'typo3/ext/'));
	}

	/**
	 * Traverses a given path recursively for *testcase.php files
	 *
	 * @param	string		$path: The path to descent from
	 * @return	array		Array of paths / filenames
	 */
	protected function traversePathForTestCases($path) {
		$extensionsArr = array();
		if (@is_dir($path))	{
			$dirs = t3lib_div::get_dirs($path);
			if (is_array($dirs))	{
				sort($dirs);
				foreach($dirs as $dirName) {
					$testsPath = $path.$dirName.'/tests/';
					if (@is_dir($testsPath) && t3lib_extMgm::isLoaded ($dirName)) {
						$testCaseFileNames = array ();
						$fileNamesArr = t3lib_div::getFilesInDir($testsPath, $extensionList='php');
						if (is_array ($fileNamesArr)) {
							foreach ($fileNamesArr as $fileName) {
								if (substr ($fileName, -12, 12) == 'testcase.php') {
									$testCaseFileNames[] = $fileName;
								}
							}

							$extensionsArr[$dirName] = array (
								'path' => $testsPath,
								'files' => $testCaseFileNames,
							);
						}
					}
				}
			}
		}
		return $extensionsArr;
	}

	/**
	 * Roughly simulates the frontend although being in the backend.
	 *
	 * @return	void
	 * @todo	This is a quick hack, needs proper implementation
	 */
	protected function simulateFrontendEnviroment() {

		global $TSFE, $TYPO3_CONF_VARS;

			// FIXME: Currently bad workaround which only initializes a few things, not really what you'd call a frontend enviroment

		require_once(PATH_tslib.'class.tslib_fe.php');
		require_once(PATH_t3lib.'class.t3lib_page.php');
		require_once(PATH_t3lib.'class.t3lib_userauth.php');
		require_once(PATH_tslib.'class.tslib_feuserauth.php');
		require_once(PATH_t3lib.'class.t3lib_tstemplate.php');
		require_once(PATH_t3lib.'class.t3lib_cs.php');

		$temp_TSFEclassName = t3lib_div::makeInstanceClassName('tslib_fe');
		$TSFE = new $temp_TSFEclassName(
				$TYPO3_CONF_VARS,
				t3lib_div::_GP('id'),
				t3lib_div::_GP('type'),
				t3lib_div::_GP('no_cache'),
				t3lib_div::_GP('cHash'),
				t3lib_div::_GP('jumpurl'),
				t3lib_div::_GP('MP'),
				t3lib_div::_GP('RDCT')
			);
		$TSFE->connectToDB();
		$TSFE->config = array();		// Must be filled with actual config!

	}
}

	// Make instance:
$SOBE = new tx_phpunit_module1;
$SOBE->init();
$SOBE->main();

?>